#ifndef PLL_ISRDISPATCH_CXX
#define PLL_ISRDISPATCH_CXX

//----------------------------------
// includes and forward declarations
//----------------------------------
#include "cyg/hal/pll_isrdispatch.h"

#if !defined(CYG_HAL_STARTUP_STUBS) && defined(CYG_HAL_MPC555)
#include "cyg/hal/mpc555_regs.h"
#include "cyg/hal/hal_io.h"
#include "cyg/hal/hal_intr.h"

//---------------------
// Class implementation
//---------------------
// Public constructors(s) and destructor
PLL_isrDispatcher::PLL_isrDispatcher()
    : HAL_isrDispatcher()
{
}

// Provide an (empty) virtual destructor
PLL_isrDispatcher::~PLL_isrDispatcher()
{
}

//---------------------------
// Protected member functions
//---------------------------
// findInterruptingVector
cyg_uint32 PLL_isrDispatcher::findInterruptingVector(void)
{
  cyg_uint16 colir;

  HAL_READ_UINT16(CYGARC_REG_IMM_COLIR, colir);
  
  if((colir & CYGARC_REG_IMM_COLIR_COLIS) && (colir & CYGARC_REG_IMM_COLIR_COLIE))
    return CYGNUM_HAL_INTERRUPT_SIU_COL;

  return 0;
}

// setInterruptingLevel
bool PLL_isrDispatcher::setInterruptingLevel(unsigned char a_level)
{
  cyg_uint16 colir;
  cyg_uint16 level = (CYGARC_REG_IMM_COLIR_IRQ0 >> a_level);

  HAL_READ_UINT16(CYGARC_REG_IMM_COLIR, colir);
  colir &= ~((cyg_uint16)CYGARC_REG_IMM_COLIR_COLIRQ);
  colir |= level;
  colir &= ~(CYGARC_REG_IMM_COLIR_COLIS); // Prevent from clearing interrupt flag accidently.
  HAL_WRITE_UINT16(CYGARC_REG_IMM_COLIR, colir);

  return true;
}

// enableInterruptingDevice
void PLL_isrDispatcher::enableInterruptingDevice(void)
{
  // Nothing to do, the interrupt can only be masked or unmasked, not disabled
}

// disableInterruptingDevice
void PLL_isrDispatcher::disableInterruptingDevice(void)
{
  // Nothing to do, the interrupt can only be masked or unmasked, not disabled
}
#endif // if !defined ...

#endif // PLL_ISRDISPATCH_CXX
