#ifndef CYGONCE_INITBOARD_C
#define CYGONCE_INITBOARD_C

//----------------------------------
// Includes and forward declarations
//----------------------------------
#include <cyg/hal/initboard.h>
#include <cyg/hal/mpc555_regs.h>
#include <pkgconf/system.h>

//-------------------------
// Functions implementation
//-------------------------
void initboard(void)
{
  SIUMCR        = 0x0000;
  SYPCR         = 0xff88;        // Note this disables the watchdog !!!!!

  // Layout of the BR registers
  // =================================================================================================
  // |   0 |   1 |   2 |   3 |   4 |   5 |   6 |   7 |   8 |   9 |  10 |  11 |  12 |  13 |  14 |  15 |
  // -------------------------------------------------------------------------------------------------
  // | Base address (BA)                                                                             |
  // -------------------------------------------------------------------------------------------------
  // | BA  | AT              | PS        |xxxxx| WP  |xxxxxxxxxxx|WEBS |TBDIP|LBDIP|SETA |  BI |  V  |
  // -------------------------------------------------------------------------------------------------
  // |  16 |  17 |  18 |  19 |  20 |  21 |  22 |  23 |  24 |  25 |  26 |  27 |  28 |  29 |  30 |  31 |
  // =================================================================================================

  // Layout of the OR registers
  // =================================================================================================
  // |   0 |   1 |   2 |   3 |   4 |   5 |   6 |   7 |   8 |   9 |  10 |  11 |  12 |  13 |  14 |  15 |
  // -------------------------------------------------------------------------------------------------
  // | Address mask (AM)                                                                             |
  // -------------------------------------------------------------------------------------------------
  // | AM  | ATM             |CSNT | ACS       |EHTR | SCY                   | BSCY            | TRLX|
  // -------------------------------------------------------------------------------------------------
  // |  16 |  17 |  18 |  19 |  20 |  21 |  22 |  23 |  24 |  25 |  26 |  27 |  28 |  29 |  30 |  31 |
  // =================================================================================================
  // Note : the base address gives the start address the chip select should be asserted to, the address
  // mask determines the end address it should be asserted to.

  // Note also, since the CME555 is a development board with lots of configuration possibilities, that 
  // the contents of this file is strongly related to the hardware layout of the board, and thus of 
  // jumper settings (Configuration as it is described here will only work for CS0 = external flash, CS1 
  // = external SRAM)

  // CS0 configuration
  BR0           = 0x00400003;      // Base address 0x400000, no burst support, 32 bit wide data port
  OR0           = 0xfff80032;      // Valid adresses till 0x47FFFF, three wait states for access time 70ns

  // CS1 configuration
  BR1           = 0x00800003;      // Base address 0x800000, no burst support, 32 bit wide data port
  OR1           = 0xfff80002;      // Valid adresses till 0x87FFFF, no wait states

  // CS2 configuration
  // Not used for now, will be used in future for the memory expansion
  // BR2           = 0xc00003;      // No burst support
  // OR2           = 0xfffe0022;

  // CS3 configuration
  BR3           = 0x01000403;       // works for keypad/LCD
  OR3           = 0xfffe0ff1;       // maximum wait states

  // And  some other configuration registers
  DMBR          = 0x00000001;       // map internal flash to 0x00400000
  DMOR          = 0x00;

  SCCR          = 0x03010000;       // Limb mode enable, TMBCLK = BUS/16, RTCCLK = Crystal / 16
  PLPRCR        = 0x900000;         // generate 40 MHz bus clock
#if defined(CYG_HAL_STARTUP_STUBS)
  TBSCR         = 0x0003;           // Enable the time base and set the freeze flag

  RTCSCK        = 0x55ccaa33;       // Unlock the RTC register set      
  RTCSC         = 0x0012;           // RTC is clocked by 4MHz crystal, set the freeze flag
  PISCR         = 0x0002;           // Set the freeze flag for the Periodic interrupt timer
#else
  TBSCR         = 0x0001;           // Enable the time base, do not respond to freeze

  RTCSCK        = 0x55ccaa33;       // Unlock the RTC register set      
  RTCSC         = 0x0010;           // RTC is clocked by 4MHz crystal, do not respond to freeze
  PISCR         = 0x0000;           // Do not respond to freeze
#endif

  DPTMCR        = 0x00;
  RAMBAR        = 0xffa0;
  PORTQS        = 0x00;
  PQSPAR_DDRQST = 0x00;
  MPIOSMDR      = 0x00;
  MPIOSMDDR     = 0x00;
  MIOS1TPCR     = 0x00;
  SRAMMCR_A     = 0x0000;
  SGPIODT1      = 0x0000;
  SGPIODT2      = 0x0000;
  SGPIOCR       = 0x0000;
  EMCR          = 0x0000;
  UMCR          = 0x0000;
}

#endif // CYGONCE_INITBOARD_C
