#ifndef MPC555_CAN_H
#define MPC555_CAN_H

//----------------------------------
// Includes and forward declarations
//----------------------------------
#include "cyg/infra/cyg_type.h"

//-------------------------
// Definitions of constants
//-------------------------
// The Toucan registermap
#define TOUCAN_A_BASE             0x307080
#define TOUCAN_B_BASE             0x307480

#define TCNMCR                    0x00
#define TTR                       0x02
#define CANICR                    0x04
#define CANCTRL0_CANCTRL1         0x06
#define PRESDIV_CTRL2             0x08
#define TIMER                     0x0a
#define RXGMASKHI                 0x10
#define RXGMASKLO                 0x12
#define RX14MASKHI                0x14
#define RX14MASKLO                0x16
#define RX15MASKHI                0x18
#define RX15MASKLO                0x1a
#define ESTAT                     0x20
#define IMASK                     0x22
#define IFLAG                     0x24
#define RXECTR_TXECTR             0x26        
#define MBUFF0                    0x80
#define MBUFF1                    0x90
#define MBUFF2                    0xa0
#define MBUFF3                    0xb0
#define MBUFF4                    0xc0
#define MBUFF5                    0xd0
#define MBUFF6                    0xe0
#define MBUFF7                    0xf0
#define MBUFF8                    0x100
#define MBUFF9                    0x110
#define MBUFF10                   0x120
#define MBUFF11                   0x130
#define MBUFF12                   0x140
#define MBUFF13                   0x150
#define MBUFF14                   0x160
#define MBUFF15                   0x170

// Bit definitions of the Toucan registers
#define TCNMCR_STOP               0x8000
#define TCNMCR_FRZ                0x4000
#define TCNMCR_HALT               0x1000
#define TCNMCR_NOTRDY             0x0800
#define TCNMCR_WAKEMSK            0x0400
#define TCNMCR_SOFTRST            0x0200
#define TCNMCR_FRZACK             0x0100
#define TCNMCR_SUPV               0x0080
#define TCNMCR_SELFWAKE           0x0040
#define TCNMCR_APS                0x0020
#define TCNMCR_STOPACK            0x0010

#define CANICR_IRL                0x0700
#define CANICR_ILBS               0x00c0
#define CANICR_IRL_SHIFT               8
#define CANICR_ILBS_SHIFT              6

#define CANCTRL0_CANCTRL1_BOFFMSK 0x8000
#define CANCTRL0_CANCTRL1_ERRMSK  0x4000
#define CANCTRL0_CANCTRL1_RXMOD   0x0c00
#define CANCTRL0_CANCTRL1_TXMOD   0x0300
#define CANCTRL0_CANCTRL1_SAMP    0x0080
#define CANCTRL0_CANCTRL1_TSYNC   0x0020
#define CANCTRL0_CANCTRL1_LBUF    0x0010
#define CANCTRL0_CANCTRL1_0D      0x0008
#define CANCTRL0_CANCTRL1_PROPSE  0x0007

#define CANCTRL0_CANCTRL1_RXMOD_SHIFT  8
#define CANCTRL0_CANCTRL1_TXMOD_SHIFT 10

#define PRESDIV_CTRL2_PRESDIV     0xff00
#define PRESDIV_CTRL2_RJW         0x00c0
#define PRESDIV_CTRL2_PSEG1       0x0038
#define PRESDIV_CTRL2_PSEG2       0x0007
#define PRESDIV_CTRL2_PRESDIV_SHIFT    8
#define PRESDIV_CTRL2_RJW_SHIFT        6
#define PRESDIV_CTRL2_PSEG1_SHIFT      3

#define RXMASK_BASE_ID            0xffe00000
#define RXMASK_EXTENDED_ID        0x0007ffff

#define ESTAT_BITER               0xc000
#define ESTAT_ACKERR              0x2000
#define ESTAT_CRCERR              0x1000
#define ESTAT_FORMERR             0x0800
#define ESTAT_STUFFERR            0x0400
#define ESTAT_TXWARN              0x0200
#define ESTAT_RXWARN              0x0100
#define ESTAT_IDLE                0x0080
#define ESTAT_TX_RX               0x0040
#define ESTAT_FCS                 0x0030
#define ESTAT_BOFFINT             0x0004
#define ESTAT_ERRINT              0x0002
#define ESTAT_WAKEINT             0x0001

#define IMASK_IMASKH              0xff00
#define IMASK_IMASKL              0x00ff

#define IFLAG_IFLAGH              0xff00
#define IFLAG_IFLAGL              0x00ff

#define RXECTR_TXECTR_RXECTR      0xff00
#define RXECTR_TXECTR_TXECTR      0x00ff

// Baud rate type definitions
typedef struct {
  unsigned char m_prescaler;    // The prescaler to be derive the S-clock from the bus clock
  unsigned char m_resync;       // The resynchronisation jump width in time quanta
  unsigned char m_pseg1;        // PSEG1 duration in time quanta
  unsigned char m_pseg2;        // PSEG2 duration in time quanta
  unsigned char m_propseg;      // PROPSEG duration in time quanta
  unsigned char m_highsample;   // use three samples per bit if set to 1
} t_can_bittiming;

// Prescaler divide register selectors
static t_can_bittiming CiA_bittiming[] = {
  {   0, 0, 0, 0, 0, 0},       // Not used
  {   4, 1, 2, 1, 1, 0},       // 1000 KBit/s = 40Mhz / (  4 + 1) / ( 3 + 1 + 0 + 4)
  {   4, 1, 0, 1, 5, 0},       // 800  KBit/s = 40Mhz / ( 4 + 1)  / ( 0 + 1 + 5 + 4)
  {   4, 1, 7, 1, 4, 1},       // 500  KBit/s = 40MHz / (  4 + 1) / (11 + 1 + 0 + 4)
  {   9, 1, 7, 1, 4, 1},       // 250  KBit/s = 40MHz / (  9 + 1) / (11 + 1 + 0 + 4)
  {  19, 1, 7, 1, 4, 1},       // 125  KBit/s = 40MHz / ( 19 + 1) / (11 + 1 + 0 + 4)
  {  24, 1, 7, 1, 4, 1},       // 100  KBit/s = 40MHz / ( 19 + 1) / (14 + 2 + 0 + 4)
  {  49, 1, 7, 1, 4, 1},       // 50   KBit/s = 40MHz / ( 39 + 1) / (14 + 2 + 0 + 4)
  { 124, 1, 7, 1, 4, 1},       // 20   KBit/s = 40MHz / ( 99 + 1) / (14 + 2 + 0 + 4)
  { 249, 1, 7, 1, 4, 1}        // 10   KBit/s = 40MHz / (199 + 1) / (14 + 2 + 0 + 4)
};

// RxMode selectors
static unsigned char RxMode[] = {
  0,
  0,
  3
};

// TxMode Selectors
static unsigned char TxMode[] = {
  0,
  0,
  1,
  3
};

// Message buffer type definitions
// Notice that only standard identifiers are supported by this driver
typedef struct {
  cyg_uint8  m_time_stamp;
  cyg_uint8  m_code          : 4;
  cyg_uint8  m_length        : 4;
  cyg_uint16 m_id            : 11; 
  cyg_uint16 m_rtr           : 1;
  cyg_uint16 m_ide           : 1;
  cyg_uint16 m_unused        : 3;
  cyg_uint16 m_time_stamp2;
  cyg_uint8  m_data_byte[8];
  cyg_uint16 m_reserved;
} t_can_messagebuffer;

// Message buffer control words
#define TOUCAN_MB_RX_DISABLE      0x0
#define TOUCAN_MB_RX_ENABLE       0x4
#define TOUCAN_MB_RX_FULL         0x2
#define TOUCAN_MB_RX_OVER         0x6
#define TOUCAN_MB_RX_BUSY         0x1
#define TOUCAN_MB_TX_DISABLE      0x8
#define TOUCAN_MB_TX_ENABLE       0xc
#define TOUCAN_MB_TX_REM_REQ      0xc
#define TOUCAN_MB_TX_REM_REP      0xa
#define TOUCAN_MB_TX_EN_REM_REP   0xe

#endif // MPC555_CAN_H
