#ifndef KTIMER_CXX
#define KTIMER_CXX

//----------------------------------
// Includes and forward declarations
//----------------------------------
#include "cyg/kal/ktimer.h"
#include "ktimerbase.h"
#include "pkgconf/kal.h"

//------------
// Static data
//------------
unsigned char highresstack[KAL_HIGHRES_TIMER_STACKSIZE];
unsigned char normalresstack[KAL_NORMALRES_TIMER_STACKSIZE];
unsigned char lowresstack[KAL_LOWRES_TIMER_STACKSIZE];

KTimerBase highbase(KAL_HIGHRES_TIMER_THREAD_PRIORITY, 
                    "High resolution timer base", 
                    &highresstack[0],
                    KAL_HIGHRES_TIMER_STACKSIZE,
                    KClockTickGenerator::m_highres) CYG_INIT_PRIORITY(APPLICATION);
KTimerBase normalbase(KAL_HIGHRES_TIMER_THREAD_PRIORITY, 
                      "Normal resolution timer base", 
                      &normalresstack[0],
                      KAL_NORMALRES_TIMER_STACKSIZE,
                      KClockTickGenerator::m_normalres) CYG_INIT_PRIORITY(APPLICATION);
KTimerBase lowbase(KAL_HIGHRES_TIMER_THREAD_PRIORITY, 
                   "Low resolution timer base", 
                   &lowresstack[0],
                   KAL_LOWRES_TIMER_STACKSIZE,
                   KClockTickGenerator::m_lowres) CYG_INIT_PRIORITY(APPLICATION);

//---------------------
// Class implementation
//---------------------
// Class KTimerClient
// Provide an (empty) virtual destructor
KTimerClient::~KTimerClient()
{
}

// Class KTimer

// Allocate memory for static members
KTimerBase * KTimer::m_highbase   = &highbase;
KTimerBase * KTimer::m_normalbase = &normalbase;
KTimerBase * KTimer::m_lowbase    = &lowbase;

// Constructor
KTimer::KTimer(KTimerClient * a_client)
    : m_schedule(KTimer::Background),
      m_precision(KTimer::Normal),
      m_client(a_client),
      m_base(m_normalbase)
{
  m_next = 0;
  m_prev = 0;

  cyg_alarm_create(m_base->getClock(),
                   (cyg_alarm_t *)stimeOut,
                   (cyg_addrword_t)this,
                   &m_handle,
                   this);
}

KTimer::KTimer(KTimerClient * a_client,
               KTimer::KTimerPrecision a_precision,
               KTimer::KTimerSchedule a_schedule)
    : m_schedule(a_schedule),
      m_precision(a_precision),
      m_client(a_client)
{
  m_next = 0;
  m_prev = 0;

  switch(a_precision)
  {
    case High:
      m_base = m_highbase;
      break;
    case Low:
      m_base = m_lowbase;
      break;
    case Normal:
    default:
      m_base = m_normalbase;
      break;
  }
  
  cyg_alarm_create(m_base->getClock(),
                   (cyg_alarm_t *)stimeOut,
                   (cyg_addrword_t)this,
                   &m_handle,
                   this);
}

// Destructor
KTimer::~KTimer()
{
  stop();
  cyg_alarm_delete(m_handle);
}

// Other public members
// start
void KTimer::start(unsigned long a_microsecs)
{
  cyg_alarm_initialize(m_handle, 
                       m_base->getTimeOut(a_microsecs), 
                       m_base->getInterval(a_microsecs));
}

// oneShot
void KTimer::oneShot(unsigned long a_microsecs)
{
  cyg_alarm_initialize(m_handle, 
                       m_base->getTimeOut(a_microsecs), 
                       0);
}

// stop
void KTimer::stop(void)
{
  cyg_alarm_disable(m_handle);
  if(m_next && m_prev)
    m_base->remove(this);
}

// stimeOut
void KTimer::stimeOut(KTimerHandle /*a_handle*/, void * a_data)
{
  switch(((KTimer *)a_data)->m_schedule)
  {
    case KTimer::Foreground:
      ((KTimer *)a_data)->timeout();
      break;
    case KTimer::Background:
      (((KTimer *)a_data)->m_base)->add((KTimer *)a_data);
      break;
    default:
      break;
  }
}

// timeout
void KTimer::timeout(void)
{
  m_client->timeOut(this);
}

// Class KTimerList
// Constructor
KTimerList::KTimerList()
    : m_head(0),
      m_tail(0)
{
  m_head.m_next = &m_tail;
  m_tail.m_prev = &m_head;
}

// Provide an (empty) virtual destructor
KTimerList::~KTimerList()
{
}

#endif // KTIMER_CXX
