#ifndef CALTST_CXX
#define CALTST_CXX

//----------------------------------
// Includes and forward declarations
//----------------------------------
#include <stdio.h>
#include <cyg/kal/ktimer.h>
#include <cyg/io/cmsevent.h>
#include <cyg/kal/kdelay.h>
#include <cyg/kal/kdefs.h>

//------------------
// Some test classes
//------------------
int ceventcount = 0;

class UTestClient : public UncontrolledClientEvent
{
  public:
                   UTestClient(can_io_handle_t a_handle)
                       : UncontrolledClientEvent(0, 5, "UTestClient", 1, a_handle, 8)
                   {
                   }
    virtual        ~UTestClient()
                   {
                   }

  protected:
    virtual void   eventDataIndication(unsigned char * a_data)
                   {
                   char str[9];
                   memcpy(str, a_data, 8);
                   str[8] = 0;
                   printf("%s\n", str);
                   }
};

class UTestServer : public UncontrolledServerEvent,
                    public KTimerClient
{
  public:
                   UTestServer(can_io_handle_t a_handle)
                       : UncontrolledServerEvent(0, 5, "UTestServer", 1, a_handle, 8),
                         KTimerClient(),
                         m_timer(this, KTimer::High, KTimer::Background)
                   {
                     m_timer.start(1000000);
                   }
    virtual        ~UTestServer()
                   {
                   }

    virtual void   timeOut(KTimer * a_timer)
                   {
                     notifyEvent((unsigned char *)"Uncontro");
                   }
  private:
    KTimer         m_timer;
};

class CTestClient : public ControlledClientEvent
{
  public:
                   CTestClient(can_io_handle_t a_handle)
                       : ControlledClientEvent(0, 5, "UtestClient", 2 ,3, a_handle, 7)
                   {
                   }
  protected:
    virtual void   eventDataIndication(unsigned char * a_data)
                   {
                   char str[8];
                   memcpy(str, a_data, 7);
                   str[7] = 0;
                   printf("%s\n", str);
                   ceventcount++;
                   }
};

class CTestServer : public ControlledServerEvent,
                    public KTimerClient
{
  public:
                   CTestServer(can_io_handle_t a_handle)
                       : ControlledServerEvent(0, 5, "UTestServer", 3, 2, a_handle, 7),
                         KTimerClient(),
                         m_timer(this, KTimer::High, KTimer::Background)
                   {
                     m_timer.start(2100000);
                   }
    virtual        ~CTestServer()
                   {
                   }

    virtual void   timeOut(KTimer * a_timer)
                   {
                     if(m_controlstate == Enabled)
                       notifyEvent((unsigned char *)"Control");
                   }

  protected:
    virtual bool   setControlState(ControlledServerEvent::ControlState a_state,
                                   unsigned char * a_response)
                   {
                     m_controlstate = a_state;
                     return true;

//                     a_response[0] = 'N';
//                     a_response[1] = 'O';
//                     a_response[2] = 'P';
//                     a_response[3] = 'E';
//                     a_response[4] = ' ';
//                     a_response[5] = 'E';
//                     a_response[6] = 'V';

//                     return false;
                   }

  private:
    KTimer         m_timer;
};

class STestClient : public StoredClientEvent
{
  public:
                   STestClient(can_io_handle_t a_handle)
                       : StoredClientEvent(0, 5, "STestClient", 4, a_handle, 8)
                   {
                   }
    virtual        ~STestClient()
                   {
                   }
  
  protected:
    virtual void   eventDataIndication(unsigned char * a_data)
                   {
                   char str[9];
                   memcpy(str, a_data, 8);
                   str[8] = 0;
                   printf("%s\n", str);
                   }
};

class STestServer : public StoredServerEvent,
                    public KTimerClient
{
  public:
                   STestServer(can_io_handle_t a_handle)
                       : StoredServerEvent(0, 5, "STestServer", 4, a_handle, 8),
                         KTimerClient(),
                         m_timer(this, KTimer::High, KTimer::Background),
                         m_count(0)
                   {
                     m_timer.start(3200000);
                   }
    virtual        ~STestServer()
                   {
                   }

    virtual void   timeOut(KTimer * a_timer)
                   {
                     m_count++;
                     char tmp[8];
                     sprintf(tmp, "%8.8d", m_count);
                     storeEvent((unsigned char *)&tmp[0], false);
                   }
    void           store(void)
                   {
                     char tmp[8];
                     sprintf(tmp, "%8.8d", m_count);
                     storeEvent((unsigned char *)&tmp[0], false);
                   }

  private:
    KTimer         m_timer;
    int            m_count;
};

int main(void)
{
  printf("Starting test\n");
  printf("=============\n");

  // Configure the canbus for 500 KBit/s 
  can_io_handle_t caldev1;
  can_io_handle_t caldev2;
  Cyg_ErrNo error;

  error = can_io_lookup("/dev/cal1", &caldev1);
  error = can_io_lookup("/dev/cal2", &caldev2);

  can_info_t info;
  DWORD len;

  info.m_baud    = CAN_BAUD_KBIT_500;
  info.m_tx_mode = CAN_TX_MODE_DEFAULT;
  info.m_rx_mode = CAN_RX_MODE_DEFAULT;
  len = sizeof(info);

  can_io_set_config(caldev1, CAN_IO_SET_CONFIG_CAN_INFO, &info, &len);
  can_io_set_config(caldev2, CAN_IO_SET_CONFIG_CAN_INFO, &info, &len);

  // Now create the CMS objects
  UTestClient uclient(caldev2);
  UTestServer userver(caldev1);
  uclient.define();
  userver.define();

  CTestClient cclient(caldev2);
  CTestServer cserver(caldev1);
  cclient.define();
  cserver.define();

  STestClient sclient(caldev2);
  STestServer sserver(caldev1);
  sclient.define();
  sserver.define();
  sserver.store();

  unsigned char response[9];
  if(cclient.setEventControlState(true, response) == -CMS_APPLICATIONERR)
    printf("%c%c%c%c%c%c%c\n", (char)response[0],
                               (char)response[1],
                               (char)response[2],
                               (char)response[3],
                               (char)response[4],
                               (char)response[5],
                               (char)response[6]);

  // And finally, start the test
  for(;;)
  {
    KDelay(500);
    if(ceventcount == 10)
    {
      if(cclient.setEventControlState(false, response) == -CMS_APPLICATIONERR)
        printf("%c%c%c%c%c%c%c\n", (char)response[0],
                                   (char)response[1],
                                   (char)response[2],
                                   (char)response[3],
                                   (char)response[4],
                                   (char)response[5],
                                   (char)response[6]);
    }

    if(sclient.readEvent(response) == CMS_NOERR)
    {
      response[8] = 0;
      printf("%s\n", response);
    }
  }

  return 0;
}

#endif // CALTST_CXX
